(function($) {
  /*
    ======== A Handy Little QUnit Reference ========
    http://api.qunitjs.com/

    Test methods:
      module(name, {[setup][ ,teardown]})
      test(name, callback)
      expect(numberOfAssertions)
      stop(increment)
      start(decrement)
    Test assertions:
      ok(value, [message])
      equal(actual, expected, [message])
      notEqual(actual, expected, [message])
      deepEqual(actual, expected, [message])
      notDeepEqual(actual, expected, [message])
      strictEqual(actual, expected, [message])
      notStrictEqual(actual, expected, [message])
      throws(block, [expected], [message])
  */
  module('PasswordMeterEngine', {
    setup: function(){
      this.rules = [
        new $.Rule($.DIGITS, 1, 'digits'),
        new $.Rule($.CHARS, 2, 'chars')
      ];
    }
  });

  test('has score 33,33333', function(){
    expect(1);
    notStrictEqual($.PasswordMeterEngine.measure('password', this.rules), 33);
  });

  test('has score 100', function(){
    expect(1);
    notStrictEqual($.PasswordMeterEngine.measure('p4ssw0rd', this.rules), 100);
  });

  test('less than 100', function(){
    expect(1);
    equal($.PasswordMeterEngine.measure('123456', this.rules) < 100, true);
  });

  test('has min lower cased', function(){
    expect(1);
    equal($.PasswordMeterEngine.lowerUpper('abcdfg', 2, 0), true);
  });

  test('has min upper case', function(){
    expect(1);
    equal($.PasswordMeterEngine.lowerUpper('ABCDFG', 0, 2), true);
  });

  test('has min and upper cases', function(){
    expect(1);
    equal($.PasswordMeterEngine.lowerUpper('ABCdefg', 2, 2), true);
  });

  test('has not min and upper cases', function(){
    expect(1);
    equal($.PasswordMeterEngine.lowerUpper('123456', 0, 0), true);
  });

  module('jQuery#passStrength', {
    // This will run before each test in this module.
    setup: function() {
      this.elems = $('#qunit-fixture').children();
    }
  });

  test('is chainable', function() {
    expect(1);
    // Not a bad test to run on collection methods.
    strictEqual(this.elems.passStrength(), this.elems, 'should be chainable');
  });

  module('jQuery.PasswordMeterEngine.conjoin', {
    setup: function(){
      this.conjoin = $.PasswordMeterEngine.conjoin;
    }
  });

  test('returns joined string', function(){
    expect(1);
    equal(this.conjoin({}, ['first string', 'second string', 'third string']), 'first string second string and third string');
  });

  test('when 5 words resturns a string', function(){
    expect(1);
    equal(this.conjoin({}, ['f', 'i', 'r', 's', 't']), 'f i r s and t');
  });

  test('when 2 words returns string', function(){
    expect(1);
    equal(this.conjoin({}, ['first', 'second']), 'first and second');
  });

  test('when 1 word returns string', function(){
    expect(1);
    equal(this.conjoin({}, ['first']), 'first');
  });

  test('returns string with delimeter', function(){
    expect(1);
    equal(this.conjoin({ delimeter: ', ', dot: '. ', last: ' or ' }, ['first', 'second', 'third']), 'first, second or third. ');
  });

  module('jQuery.PasswordMeterEngine.repetition');
  test('check string has repetitions', function(){
    expect(1);
    equal($.PasswordMeterEngine.repetitions(1, 'tt'), true);
  });

  test('check string has no repetitions', function(){
    expect(1);
    equal($.PasswordMeterEngine.repetitions(1, 'ta'), false);
  });

  test('check string has two groups of repetitions', function(){
    expect(1);
    equal($.PasswordMeterEngine.repetitions(2, 'tttta'), true);
  });
}(jQuery));
